# -*- coding: utf-8 -*-
"""
Created on Mon Apr 27 23:15:37 2020

@author: 35732
"""
from configparser import ConfigParser
class Dictionary:
    def __init__(self,path):#表示配置文件的路径
        cp = ConfigParser()
        cp.read(path,'UTF-8')
        self.replace_word = eval(cp.get("DICTIONARY",'replace_word'))
        self.alloy_to_replace = eval(cp.get("DICTIONARY",'alloy_to_replace'))
        self.paras_to_replace = eval(cp.get("DICTIONARY",'paras_to_replace'))
        self.alloy_writing_type = eval(cp.get("DICTIONARY",'alloy_writing_type'))
        self.alloy_blank_type = eval(cp.get("DICTIONARY",'alloy_blank_type'))
        self.prop_writing_type = eval(cp.get("DICTIONARY",'prop_writing_type'))
        self.value_wt = eval(cp.get("DICTIONARY",'value_wt'))
        self.other_phase = eval(cp.get("DICTIONARY",'other_phase'))
        self.unit_replace = eval(cp.get("DICTIONARY",'unit_replace'))
        self.no_unit_para = eval(cp.get("DICTIONARY",'no_unit_para'))
        self.other_quality = eval(cp.get("DICTIONARY",'other_quality'))
        
        
        
        
        
        
        
#        self.replace_word = {'Fig .': 'fig ', 'Fig.': 'fig', 'fig.': 'fig', 'et al.': 'et al', '℃': '°C', '˚C': '°C', 'g cm−3': 'g/cm3', 'gm/cc': 'g/cm3', 'g cm3': 'g/cm3', '–': '-', 'L12': 'γ′', 'incipient melting': 'incipient-melting', "γ'": 'γ′', 'gamma prime': 'γ′', '™': 'TM', '<': '＜', '>': '＞', '＞ ': '＞', ' ＜': '＜', 'solution heat': 'solution-heat'}
#        self.alloy_to_replace = {r'\s([A-Z]+[a-z]*\s+[0-9]+)\s':["\s+","~"]}#会与很多仪器名称重叠
#        
#        self.paras_to_replace = {"\s[0-9]+\s+C":["\s+C","°C"],"[0-9]+\s+°C":["\s+",""],"[0-9]+\s+K":["\s+K","K"],'\s+±\s+':['\s+',''],'\s+–\s+':['\s+',''],'[0-9]+\s+g/cm3':['\s+g/cm3','g/cm3'],'\d\s+%':['\s+%','/%/']}
#        
#        self.alloy_writing_type = [r'^[0-9]+\.?[0-9]{0,2}[A-JL-Z]',r'[0-9]{0,2}\.?[0-9]{0,2}[A-Z][a-z]?\-[0-9]{0,2}\.?[0-9]{0,2}[A-Z][a-z]?',r'^[A-Z]+[a-z]*\-\S*[0-9]+$',r'^[A-Z]\S+[0-9]$',r'^[A-Z]+\S*\-[0-9A-Z]\S*',r'^[A-Z]+[0-9]+[A-z]+',r'^[A-Z]+[a-z]*\~[A-Z0-9]+$']
#        
#        self.alloy_blank_type = ['\s([0-9A-Z]+\w*)\s+\S*[Aa]lloy\s','\s+\S*[Aa]lloy\s+([0-9A-Z]+\w*)\s']
#        #用\w而不用\s是为了避免匹配Co-base等名称
#        #density中+s?
#        
#        self.prop_writing_type = {'solidus':['Tsolidus','Solidus','solidus'],'solvus':['γ′-solvus','γ′Solvus','solvus','γ′-transus','Tγ′','solvus-temperature','dissolution'],'density':['density','densities','densityρ','pO-'],'hardness':['microhardness', 'micro-hardness', 'nanohardness','hardnesses','nano-hardness','microhardnesses','Hardness','Micro-hardness','Microhardness','harness','macrohardness'],'liquidus':['liquidus','Liquidus','Tlq','liquidous','liquidusus']}
#        
#        self.value_wt = {'solidus':[r'^\W{0,1}[7-9][0-9]{2}(\.[0-9]{1,2})?\S*°C$','^\W{0,1}[7-9][0-9]{2}(\.[0-9]{1,2})?\S*K$','^\W{0,1}1[0-9][0-9]{2}\S*°C$','^\W{0,1}1[0-9][0-9]{2}\S*K$'],'solvus':[r'^\W{0,1}[7-9][0-9]{2}(\.[0-9]{1,2})?\S*°C$','^\W{0,1}[7-9][0-9]{2}(\.[0-9]{1,2})?\S*K$','^\W{0,1}1[0-9][0-9]{2}\S*°C$','^\W{0,1}1[0-9][0-9]{2}\S*K$'],'density':[r"^\W?[4-9](\.[0-9]{1,2})?\S*g/cm3|^\W?1[0,1](\.[0-9]{1,2})?\S*g/cm3"],'liquidus':[r'^\W{0,1}[7-9][0-9]{2}(\.[0-9]{1,2})?\S*°C$','^\W{0,1}[7-9][0-9]{2}(\.[0-9]{1,2})?\S*K$','^\W{0,1}1[0-9][0-9]{2}\S*°C$','^\W{0,1}1[0-9][0-9]{2}\S*K$']}
#        
##        self.triple_e_replace = {'Fig .':'fig ','Fig.':'fig','fig.':'fig','et al.':'et al',"℃":"°C","˚C":"°C",'g cm−3':'g/cm3','gm/cc':'g/cm3','L12':'γ′','incipient melting':'incipient-melting',"γ'":'γ′','gamma prime':'γ′','™':'TM','<':'＜','>':'＞','DSC':'dsc'}#各个性能分开
#        
##        self.triple_para_replace = {}
#        self.other_phase = {'solvus':['γ″','δ','η','TCP',' μ ',' P ',' R ','D019','M23 C6'],'solidus':[],'liquidus':[],'density':[]}#当这些字符出现时性能参数匹配会失败，参数改变了原来的归属,加空格是为了避免匹配到其它字符
#        self.unit_replace = {'solidus':['°C','K'],'solvus':['°C','K'],'density':['g/cm3'],'yield':['MPa'],'liquidus':['°C','K']}#是否某个性能存在两个有数量关系且通用的单位
#        self.no_unit_para = {'solvus':[r'^\W{0,1}[7-9][0-9]\S*\d$',r'^\W{0,1}1[0-9][0-9]\S*\d$'],'solidus':[r'^\W{0,1}[7-9][0-9]\S*\d$',r'^\W{0,1}1[0-9][0-9]\S*\d$'],'density':[r"^\W?[4-9]\.[0-9]{1,2}(\S\d\.\d{2})?$|^\W?1[0,1]\.[0-9]{1,2}(\S\d\.\d{2})?$"],'yield':[r'^\S?[1]?[2-9]\d{2}\.[0-9]{1,2}\S*'],'liquidus':[r'^\W{0,1}[7-9][0-9]\S*\d$',r'^\W{0,1}1[0-9][0-9]\S*\d$']}
#        self.other_quality = {'solvus':['γ′-solution','solution','solutioning','solidus','Tsolidus','Solidus','liquidus','Liquidus','Tlq','liquidous','liquidusus','incipient','age','aged','aging','heat','solution-heat','ageing','heated','heating','homogenized','homogenization','heat-treated','super-solvus','sub-solvus','γ″','solidification','eutectic','solubility','annealing','annealed','anneal','deformed','melting','incipient-melting'],'solidus':['γ′-solution','solution','solutioning','solvus','γ′-transus','liquidus','Liquidus','Tlq','liquidous','liquidusus','incipient','age','aged','aging','heat','solution-heat','ageing','heated','heating','homogenized','homogenization','heat-treated','super-solvus','sub-solvus','γ″','solidification','eutectic','solubility','annealing','annealed','anneal','deformed','melting','incipient-melting'],'density':[],'liquidus':['Tsolidus','Solidus','γ′-solution','solution','solutioning','solidus','incipient','γ′-solvus','γ′Solvus','solvus','γ′-transus','Tγ′','solvus-temperature','dissolution','age','aged','aging','heat','solution-heat','ageing','heated','heating','homogenized','homogenization','heat-treated','super-solvus','sub-solvus','γ″','solidification','eutectic','solubility','annealing','annealed','anneal','deformed','melting','incipient-melting']}#该范围的数值可能是其它的性能
        
        
        
         
        


    
    




# -*- coding: utf-8 -*-
"""
Created on Mon May 18 09:17:29 2020

@author: 35732
"""
import nltk 
import re
from dictionary import Dictionary
class Phrase_parse:
    def __init__(self,txt,prop_name,C_path):
        self.C_path = C_path
        self.dict_info = Dictionary(self.C_path)
        self.txt = txt
        self.prop_name = prop_name
    def alloy_sub_search(self):
        txt = self.txt
        alloy_write_type = self.dict_info.alloy_writing_type
        alloy_blank_type = self.dict_info.alloy_blank_type
        value_writing_type = self.dict_info.value_wt[self.prop_name]
        outcome_filter = {}
        word_list = nltk.word_tokenize(txt)
        word_id = {}
        s=0
        for word in word_list:
            word_id[word] = s
            s = s + 1
        id_word = {value:key for key,value in word_id.items()}
        sub_id = {}#句中所有主语的名称-index
        sub_order =[]
        for pattern in alloy_blank_type:
            outcome = re.findall(pattern,txt)
            if outcome:
                for name in outcome:
                    sub_number_id = word_id[name]
                    sub_id[name] = sub_number_id
                
        len_type = len(alloy_write_type)
        for word in word_list:
            for i in range(0,len_type):
                outcome_filter[i] = re.findall(alloy_write_type[i],word)
            for k in range(0,len_type):
                if outcome_filter[k] :
                    id_word = word_id[word]#主语的id
                    sub_id[word] = id_word
                    break
        name_order=sorted(sub_id.items(),key=lambda x:x[1],reverse=False)
        for item in name_order:
            sub_order.append(item[0])
        len_type = len(value_writing_type)

        outcome_filter = {}
        object_list = []
        for word in word_list:
            for t in range(0,len_type):
                outcome_filter[t] = re.findall(value_writing_type[t],word)
            for q in range(0,len_type):
                if outcome_filter[q]:
                    object_list.append(word)
                    break
        return sub_order,sub_id,object_list
    
    

    


# -*- coding: utf-8 -*-
"""
Created on Mon May 18 09:15:57 2020

@author: 35732
"""
import nltk 
import re
import copy
from dictionary import Dictionary
class Relation_extraciton:
    def __init__(self,prop_name,data,sub_order,sub_id,object_list,C_path):
        self.C_path = C_path
        self.prop_name = prop_name
        self.data = data
        self.sub_order = sub_order
        self.sub_id = sub_id
        self.object_list = object_list
        self.dictionary = Dictionary(self.C_path)
    def triple_extraction(self):
        object_list = self.object_list
        sub_order = self.sub_order
        txt = self.data
        id_word = {}
        outcome = {}
        word_list = nltk.word_tokenize(txt)
        new_word_list = []
        word_id = {}
        replace_sub =[]
        w = 0
        if all(word not in txt for word in self.dictionary.other_phase[self.prop_name]):
            for word in word_list:
                search = re.findall(r'\S*[Aa]lloy$',word)#考察density时加上s?
                if search:
                    replace_sub.append(word)

                id_word[w] = word
                w = w + 1
                new_word_list.append(word)

            new_sent = (' ').join(new_word_list)
            for s in range(0,len(sub_order)):
                if sub_order[s] not in new_sent:
                    sub_order.remove(sub_order[s])
            word_id = {value:key for key,value in id_word.items()}
            other_quality = self.dictionary.other_quality[self.prop_name]
            length_subject = len(sub_order)
            length_object = len(object_list)
            if length_subject == 0 and length_object ==1 and replace_sub:
                if any(key in word_list for key in other_quality) and replace_sub:
                    all_quality = []
                    len_quality = []
                    other_quality.append(self.prop_name)
                    for kek in other_quality:
                       if kek in word_list:
                           all_quality.append(kek)
                    for quality in all_quality:
                        quality_id = word_id[quality]
                        word = object_list[0]
                        value_id = word_id[word]
                        len_1 = abs(quality_id - value_id)
                        len_quality.append(len_1)
                    id_min = len_quality.index(min(len_quality))
                    min_quality = all_quality[id_min]
                    outcome[1] = replace_sub[0],min_quality,object_list[0]#将第一个代词作为本句的合金名称
                    other_quality.remove(self.prop_name)
                if all(key not in word_list for key in other_quality) and replace_sub:
                    outcome[1] = replace_sub[0],self.prop_name,object_list[0]
        
            if length_subject == 0 and length_object > 1 and replace_sub:

                if all(key not in word_list for key in other_quality) :                  
                    quality_id = word_id[self.prop_name]
                    distance_os = []
                    for word_obj in object_list:
                        id_word_obj = word_id[word_obj]
                        length_os =  abs(id_word_obj - quality_id)
                        distance_os.append(length_os)
                    id_min = distance_os.index(min(distance_os))
                    id_min_2 = id_min + 1
                    if id_min_2 != len(distance_os):
                        if distance_os[id_min+1] == min(distance_os):
                            id_min = id_min_2
                            outcome[1] = replace_sub[0],self.prop_name,object_list[id_min]
                        else:
                            outcome[1] = replace_sub[0],self.prop_name,object_list[id_min]  
                    else:
                        outcome[1] = replace_sub[0],self.prop_name,object_list[id_min]
                        
                if any(key in word_list for key in other_quality):
                    
                    other_quality.append(self.prop_name)
                    all_quality =[]
                    for word in word_list:
                        if word in other_quality and word not in all_quality:
                            all_quality.append(word)
                    k = 1
                    if len(all_quality) == len(object_list):
                        
                        for i in range(len(all_quality)):
                            outcome[k] = replace_sub[0],all_quality[i],object_list[i]
                            k =+ 1
                    else:
                        for word_obj in object_list:
                            id_word_qual = word_id[word_obj]
                            distance_os = []
                            for quality in all_quality:
                                id_word_obj = word_id[quality]
                                length_os = abs(id_word_qual - id_word_obj)
                                distance_os.append(length_os)
                            id_min = distance_os.index(min(distance_os))
                            outcome[k] = replace_sub[0],all_quality[id_min],word_obj
                            k += 1
                    other_quality.remove(self.prop_name)
                
            if length_subject != 0 and length_object != 0:#对于单一主语，但宾语不是溶解温度的情况
                if length_subject == length_object and length_subject == 1:
                    if any(key in word_list for key in other_quality):
                        out_id = {}
                        solidus_id = word_id[self.prop_name]
                        word = object_list[0]
                        value_id = word_id[word]
                        len_1 = abs(solidus_id - value_id)
                        for kek in other_quality:
                            if kek in word_list:
                                kek_id = word_id[kek]
                                len_2 = abs(kek_id - value_id)
                                out_id[kek] = len_2
                        out_id = sorted(out_id.items(),key=lambda x:x[1],reverse=False)
                        kek = out_id[0][0]
                        out_values = out_id[0][1]
        
                        len_min = min(out_values,len_1)
                        if len_min == len_1 :
                            outcome[1] = sub_order[0],self.prop_name,object_list[0]
                        if len_min == out_values:
                            outcome[1] = sub_order[0],kek,object_list[0]

                    else:  
                        outcome[1] = sub_order[0],self.prop_name,object_list[0]
            
                if length_subject == length_object and length_subject > 1:
                    if all(key not in new_word_list for key in other_quality):
                        for q in range(0,length_subject):
                            subject = sub_order[q]
                            obj_all = object_list[q]
                            outcome[q+1] = subject,self.prop_name,obj_all 
                    if any(key in word_list for key in other_quality):
                        all_quality =[]
                        all_quality.append(self.prop_name)
                        k = 1
                        for word in word_list:
                            if word in other_quality and word not in all_quality:
                                all_quality.append(word)
                        for word_sub in sub_order:
                            id_word_sub = word_id[word_sub]
                            distance_os = []
                            for word_obj in object_list:
                                id_obj = word_id[word_obj]
                                length_os =  abs(id_word_sub - id_obj)
                                distance_os.append(length_os)
                            id_min = distance_os.index(min(distance_os))
                            out_obj = object_list[id_min]
                            id_out_obj = word_id[out_obj]
                            distance_oq = []
                            for quality in all_quality:
                                id_quality = word_id[quality]
                                length_oq = abs(id_quality - id_out_obj)
                                distance_oq.append(length_oq)
                            id_min = distance_oq.index(min(distance_oq))
                            out_quality = all_quality[id_min]
                            outcome[k] = word_sub,out_quality,out_obj
                            k += 1

                if length_subject > length_object:
                    if all(key not in word_list for key in other_quality):
                        k = 1

                        for word_obj in object_list:
                            id_word_obj = word_id[word_obj]
                            distance_os = []
                            for word_sub in sub_order:
                                id_word_sub = word_id[word_sub]
                                length_os =  abs(id_word_obj - id_word_sub)
                                distance_os.append(length_os)
                            id_min = distance_os.index(min(distance_os))
                            subject = sub_order[id_min]
                            word_obj = word_obj 
                            outcome[k] =subject,self.prop_name,word_obj
                            k += 1

                    if any(key in word_list for key in other_quality):
                        all_quality =[]
                        all_quality.append(self.prop_name)
                        k = 1
                        for word in word_list:
                            if word in other_quality and word not in all_quality:
                                all_quality.append(word)
                        for word_obj in object_list:
                            id_word_obj = word_id[word_obj]
                            distance_oq = []

                            for quality in all_quality:
                                id_quality = word_id[quality]
                                length_oq =  abs(id_word_obj - id_quality)
                                distance_oq.append(length_oq)
                            id_min = distance_oq.index(min(distance_oq))
                            out_quality = all_quality[id_min]
                            id_out_qual = word_id[out_quality]
                            distance_sq = []
                            for word_sub in sub_order:
                                id_word_sub = word_id[word_sub]
                                length_sq =  abs(id_out_qual - id_word_sub)
                                distance_sq.append(length_sq)
                            id_min = distance_sq.index(min(distance_sq))
                            out_sub = sub_order[id_min]
                            outcome[k] = out_sub,out_quality,word_obj
                            k += 1

                if length_subject < length_object and length_subject == 1:
                    if all(key not in word_list for key in other_quality):

                        id_word_quality = word_id[sub_order[0]]
                        distance_os = []
                        for word_obj in object_list:
                            id_word_obj = word_id[word_obj]
                            length_os = abs(id_word_quality - id_word_obj)
                            distance_os.append(length_os)
                        id_min = distance_os.index(min(distance_os))
                        id_min_0 = id_min + 1
                        if id_min_0 != len(distance_os):
                            if distance_os[id_min + 1] == min(distance_os):
                                id_min = id_min + 1#防止宾语已经在列表最后而运行出错,如果两个数值距离一个主语距离相同，我们取后边一个
        
                        object_0 = object_list[id_min]
                        outcome[1] = sub_order[0],self.prop_name,object_0
                        
                    if any(key in word_list for key in other_quality):
                        
                        all_quality =[]
                        other_quality.append(self.prop_name)
                        k = 1
                        for word in word_list:
                            if word in other_quality and word not in all_quality:
                                all_quality.append(word)
                        if len(all_quality) == len(object_list):
                            for i in range(len(all_quality)):
                                outcome[k] = sub_order[0],all_quality[i],object_list[i]
                                k += 1
                        else:
                            for word_obj in object_list:
                                id_word_qual = word_id[word_obj]
                                distance_os = []
                                for quality in all_quality:
                                    id_word_obj = word_id[quality]
                                    length_os = abs(id_word_qual - id_word_obj)
                                    distance_os.append(length_os)
                                id_min = distance_os.index(min(distance_os))
                                outcome[k] = sub_order[0],all_quality[id_min],word_obj
                                k += 1
    
                        other_quality.remove(self.prop_name)
            
                if length_subject < length_object and length_subject > 1:
                    if all(key not in word_list for key in other_quality):
                        k = 1
                        for sub in sub_order:
                            id_sub = self.sub_id[sub]
                            distance_os = []
                            for word_obj in object_list:
                                id_word_obj = word_id[word_obj]
                                length_os = abs(id_sub - id_word_obj)
                                distance_os.append(length_os)
                            id_min = distance_os.index(min(distance_os))
                            id_min_0 = id_min + 1
                            if id_min_0 != len(distance_os):
                                if distance_os[id_min + 1] == min(distance_os):
                                    id_min = id_min + 1
                            object_0 = object_list[id_min]
                            outcome[k] = sub,self.prop_name,object_0
                            k += 1
                    if any(key in word_list for key in other_quality):
                        all_quality =[]
                        all_quality.append(self.prop_name)
                        k = 1
                        for word in word_list:
                            if word in other_quality and word not in all_quality:
                                all_quality.append(word)
                        for word_sub in sub_order:
                            id_word_sub = word_id[word_sub]
                            distance_so = []
                            for word_obj in object_list:
                                id_word_obj = word_id[word_obj]
                                length_so =  abs(id_word_sub - id_word_obj)
                                distance_so.append(length_so)
                            id_min = distance_so.index(min(distance_so))
                            out_obj = object_list[id_min]
                            id_out_obj = word_id[out_obj]
                            distance_oq = []    
                            for quality in all_quality:
                                id_quality = word_id[quality]
                                length_oq =  abs(id_out_obj - id_quality)
                                distance_oq.append(length_oq)
                            id_min = distance_oq.index(min(distance_oq))
                            out_quality = all_quality[id_min]
                            outcome[k] = word_sub,out_quality,out_obj
                            k += 1

        return outcome


# -*- coding: utf-8 -*-
"""
Created on Thu May 21 20:09:04 2020

@author: 35732
"""
import xlrd
import xlwt
class File_IO:
    def __init__(self,target_sents,out_path,txt_name):
        self.target_sents = target_sents
        self.out_path = out_path
        self.txt_name = txt_name
    #get_sentences
    def out_to_excel(self):
        length_sent = len(self.target_sents)
        xls = xlwt.Workbook()
        sht1 = xls.add_sheet("Sheet1")
        for w in range(0,length_sent):
            sht1.write(w,0,str(w)+'.txt')
            #这里的列索引数要与后续的一致
            if self.target_sents[w] == '{}':
                out = 'no target sentence'
                sht1.write(w,1,out)
            else:
                sht1.write(w,1,self.target_sents[w])
        xls.save(self.out_path)
    def data_from_excel(self):#k为该列的长度,原文件需要是字典的形式
        all_data = []
        file = xlrd.open_workbook(self.out_path)
        sheet = file.sheet_by_index(0)
        col_value = sheet.col_values(1)#这里的列索引数要与上面的一致
        k = len(col_value)
        
        for k in range(0,k):
            unit_data = []
            str_sent = col_value[k]
            if str_sent == 'no target sentence':
                all_data.append([])
            else:
                dict_sent = eval(str_sent)
                for i,sent in dict_sent.items():
                    unit_data.append(sent)
                all_data.append(unit_data)
        return all_data#[[],[],..]的输出形式
    



# -*- coding: utf-8 -*-
"""
Created on Wed Jul 29 09:23:52 2020

@author: 35732
"""

from __future__ import print_function
from setuptools import setup, find_packages
import sys
import setuptools
 
setup(
      name="SuperalloyDataExtractor",
      version="0.0.3",
      author="Weiren_Wang",
      author_email="357329191@qq.com",
      description="A data extractor to get target information from superalloy documents. The functions include batch downloading documents in XML and TXT format from the Elsevier database, locating target sentences from the full text and automatically extracting triple information in the form of <material name, property specifier, value>.",
      long_description=open("README.md",encoding="utf-8").read(),
      keywords='text-mining, mining, superalloy, informatics, nlp, txt, science, scientific',
      license="MIT",
      url="https://github.com/Weiren1996/superalloydataextractor",
      packages=setuptools.find_packages(),
      long_description_content_type="text/markdown",
      classifiers=[
              "Environment :: Web Environment",
              "Intended Audience :: Science/Research",
              "Operating System :: Microsoft :: Windows",
              "Programming Language :: Python :: 3",
              "Topic :: Internet",
              "Topic :: Scientific/Engineering",
              "Topic :: Text Processing",
              ],
      python_requires='>=3.6'
)


# -*- coding: utf-8 -*-
"""
Created on Tue Oct 22 22:02:21 2019

@author: 35732
"""

import os
import nltk

class Filter_text:
    #in_path为输入的文件夹路径，out_path为输出的文件夹路径

    def __init__(self,in_path,out_path):
        self.in_path = in_path
        self.out_path = out_path

    def data_totxt(sample,path):
        f = open(path,'w',encoding='utf-8')
        f.write(sample)
        f.close()
        
    def process(self):
        txt_name = os.listdir(self.in_path)
        print(txt_name)
        lengthen_0 = len(txt_name)  #计算单个文件夹下子文件的数目
        p = 0
        for k in range(0,lengthen_0):
            new_data = []
            datas_outcome = []
            file =open(self.in_path +'/'+ txt_name[k],'r',encoding='utf-8')
            data_i = file.read()
            data_i = data_i.replace('Key words','Keywords')
            data_i = data_i.replace('Keywords','Keyword')
            data_i = data_i.replace('INTRODUCTION','Introduction')
            
            token_data = nltk.word_tokenize(data_i)
            if "Keyword" in token_data:
                index_abstract = token_data.index('Keyword')
                token_data = token_data[index_abstract:]
                new_data = token_data
        
                if 'References' in new_data or 'Reference' in new_data:
                    if 'References' in new_data:
                        new_data = new_data[::-1]
                        index_References = new_data.index('References')
                        new_data = new_data[index_References:]
                        new_data = new_data[::-1]
                        datas_outcome = " ".join(new_data)
                        path = self.out_path +'\\'+ str(p) + '.txt'
                        #path = str(i)+".xml"
                        Filter_text.data_totxt(datas_outcome,path)
                        p = p+1

                    else:
                        if 'Reference' in new_data:
                            new_data = new_data[::-1]
                            index_Reference = new_data.index('Reference')
                            new_data = new_data[index_Reference:]
                            new_data = new_data[::-1]
                            datas_outcome = " ".join(new_data)
                            path = self.out_path +'/'+ str(p) + '.txt'
                            #path = str(i)+".xml"
                            Filter_text.data_totxt(datas_outcome,path)
                            p = p+1
                else:
                    datas_outcome = " ".join(new_data)
                    path = self.out_path +'/'+str(p) +'.txt'
                    #path = str(i)+".xml"
                    Filter_text.data_totxt(datas_outcome,path)
                    p = p+1
        
            elif "Introduction" in token_data:
                re_token_data = token_data[::-1]
                index_Introduction = re_token_data.index('Introduction')
                token_data = re_token_data[:index_Introduction]
                new_data = token_data[::-1]
        
                if 'References' in new_data or 'Reference' in new_data:
                    if 'References' in new_data:
                        new_data = new_data[::-1]
                        index_References = new_data.index('References')
                        new_data = new_data[index_References:]
                        new_data = new_data[::-1]
                        datas_outcome = " ".join(new_data)
                        path = self.out_path +'/'+str(p)+'.txt'
                        #path = str(i)+".xml"
                        Filter_text.data_totxt(datas_outcome,path)
                        p = p+1

                    else:
                        if 'Reference' in new_data:
                            new_data = new_data[::-1]
                            index_Reference = new_data.index('Reference')
                            new_data = new_data[index_Reference:]
                            new_data = new_data[::-1]
                            datas_outcome = " ".join(new_data)
                            path = self.out_path +'/'+str(p)+'.txt'
                            #path = str(i)+".xml"
                            Filter_text.data_totxt(datas_outcome,path)
                            p = p+1

                else:
                    datas_outcome = " ".join(new_data)
                    path = self.out_path+'/'+str(p)+'.txt'
                    #path = str(i)+".xml"
                    Filter_text.data_totxt(datas_outcome,path)
                    p = p+1

            else:
                datas_outcome = data_i
                path = self.out_path+'/'+str(p)+'.txt'
                #path = str(i)+".xml"
                Filter_text.data_totxt(datas_outcome,path)
                p = p+1
        return txt_name


